const state = {
    journals: [],
    loadingJournals: false,
    selectedJournal: null,
    showBankPaymentModal: false
}

const getters = {
    /**
     * 
     * @param {*} state 
     */
    journals(state) {
        return state.journals
    },
    /**
     * 
     * @param {*} state 
     */
    loadingJournals(state) {
        return state.loadingJournals
    },
    /**
     * 
     * @param {*} state 
     */
    selectedJournal(state) {
        return state.selectedJournal
    },
    /**
     * 
     * @param {*} state 
     */
    showBankPaymentModal(state) {
        return state.showBankPaymentModal
    }
}

const mutations = {
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setJournals (state, payload) {
        state.journals = payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setLoadingJournals (state, payload) {
        state.loadingJournals = !!payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    autoSelectJournal (state) {
        state.selectedJournal = state.journals.find(j => j.type === 'cash')
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setSelectedJournal (state, payload) {
        if (!payload) {
            state.selectedJournal = payload
            return
        }

        state.selectedJournal = state.journals.find(item => item.id === payload)

        if (state.selectedJournal.type && state.selectedJournal.type === 'bank') {
            state.showBankPaymentModal = true;
        }

        if (state.selectedJournal.type && state.selectedJournal.type === 'cash') {
            state.showBankPaymentModal = false;
        }
    },
    /**
     * 
     * @param {*} state 
     */
    setShowBankPaymentModal (state, payload) {
        state.showBankPaymentModal = !!payload
    }
}

const actions = {
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    initJournals ({ commit }, payload) {
        commit('setJournals', payload)
        commit('autoSelectJournal')
        commit('setLoadingJournals')
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    selectJournal ({ commit }, payload) {
        commit('setSelectedJournal', payload)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    submitBankPayment({ commit }, payload) {
        if (!payload.bank) {
            commit('autoSelectJournal')
            commit('setShowBankPaymentModal', false)
            return
        }
        
        commit('setSelectedBank', payload.bank)
        commit('setSelectedBankPaymentType', payload.bankType)
        commit('setBankOperationNumber', payload.bankNumber)
        commit('setBankDueDate', payload.dueDate)
        commit('setAccountBankNumber', payload.accountNumber)
        commit('setHolderBankName', payload.holder)

        commit('setShowBankPaymentModal', false)
    },
    /**
     * 
     * @param {*} param0 
     */
    resetJournal ({ commit }) {
        commit('setLoadingJournals', true)
        commit('setJournals', [])
        commit('setSelectedJournal', null)
        commit('setShowBankPaymentModal', false)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}